package tutorial.jsf.ajax;

import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.*;

/**
 * <b>Title:</b> AjaxComponent<br>
 * @author Matt Tyson
 * <p>
 * A simple phase listener to filter Ajax requests.
 */

public class AjaxListener implements PhaseListener {
	private static final transient Log log = LogFactory.getLog(tutorial.jsf.ajax.AjaxListener.class);
	// These constants are used to get values from the request params
	// Determining if the request is an AjaxComponent and what the client id of the component
	// making the request is, respectively.
	private static final String AJAX_PARAM_KEY = "tutorial.jsf.ajax.AJAX_REQUEST";
	private static final String AJAX_CLIENT_ID_KEY = "tutorial.jsf.ajax.AJAX_CLIENT_ID";
	
	/**default empty constructor*/
	public AjaxListener() {
	}

	/**
	 * Handling the any potential Ajax component requests after the Restore View phase makes the restored view
	 * available to us.  Therefore, we can get the component (which made the request) from the view,
	 * and let it respond to the request. 
	 */
	public void afterPhase(PhaseEvent event) {
		if (log.isInfoEnabled()) { log.info("BEGIN afterPhase()"); }
		FacesContext context = event.getFacesContext().getCurrentInstance();		
		
		HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();
		String ajaxParam = request.getParameter(AJAX_PARAM_KEY);
		
		// Check for the existence of the Ajax param
		if (ajaxParam != null && ajaxParam.equals("true")){
			if (log.isInfoEnabled()) { log.info("This is an ajax request."); }
			context.responseComplete(); // Let JSF know to skip the rest of the lifecycle
			
			String componentId = request.getParameter(AJAX_CLIENT_ID_KEY);  // Get the Ajax component ID
			if (componentId == null){
				if (log.isWarnEnabled()) { log.warn("No Client ID found under key: " + componentId); }
			} else {
				handleAjaxRequest(context, componentId);
			}
			
			// Save the state of the page
			context.getApplication().getStateManager().saveSerializedView(context);
		}
	}
	
	protected void handleAjaxRequest(FacesContext context, String ajaxClientId) {
		UIViewRoot viewRoot = context.getViewRoot();
		
		AjaxInterface ajaxComponent = null;
		try {
			ajaxComponent = (AjaxInterface)viewRoot.findComponent(ajaxClientId);
		} catch (ClassCastException cce){
			throw new IllegalArgumentException("Component found under Ajax key was not of expected type.");
		}
		if (ajaxComponent == null){
			throw new NullPointerException("No component found under specified client id: " + ajaxClientId);
		}
		
		ajaxComponent.handleAjaxRequest(context);
	}

	public void beforePhase(PhaseEvent arg0) {
		// We do nothing in the before phase.
	}

	public PhaseId getPhaseId() {
		return PhaseId.RESTORE_VIEW;
	}
}
